from davidkhala.utils.http_request import Request

class API(Request):
    def __init__(self, api_key: str, base_url="https://api.dify.ai/v1"):
        super().__init__({'bearer': api_key})
        self.base_url = base_url
        self.api_key = api_key


from enum import Enum
from http import HTTPStatus

from dashscope.api_entities.dashscope_response import DashScopeAPIResponse

from dashscope import Generation, TextEmbedding
from davidkhala.ai.model import AbstractClient


class ModelEnum(str, Enum):
    BAILIAN = Generation.Models.bailian_v1
    DOLLY = Generation.Models.dolly_12b_v2
    TURBO = Generation.Models.qwen_turbo
    PLUS = Generation.Models.qwen_plus
    MAX = Generation.Models.qwen_max
    EMBED = TextEmbedding.Models.text_embedding_v4


class API(AbstractClient):
    """
    Unsupported to use international base_url "https://dashscope-intl.aliyuncs.com"
    """

    model: ModelEnum

    def __init__(self, api_key):
        self.api_key = api_key

    def as_embeddings(self, model=ModelEnum.EMBED):
        super().as_embeddings(model)

    @staticmethod
    def _on_response(response:DashScopeAPIResponse):
        if response.status_code == HTTPStatus.OK:
            return response.output
        else:
            raise Exception(response)


    def chat(self, user_prompt: str, **kwargs):

        if not self.messages:
            kwargs['prompt'] = user_prompt
        else:
            kwargs['messages'] = [
                *self.messages,
                {
                    "role": "user",
                    'content': user_prompt
                }
            ]
        # prompt 和 messages 是互斥的参数：如果你使用了 messages，就不要再传 prompt
        r = Generation.call(
            self.model,
            api_key=self.api_key,
            **kwargs
        )
        return API._on_response(r)

    def encode(self, *_input: str)-> list[list[float]]:
        r= TextEmbedding.call(
            self.model,list(_input),
            api_key= self.api_key,
        )
        r = API._on_response(r)

        return [item['embedding'] for item in r['embeddings']]


import os

from davidkhala.ai.huggingface import clone
from pathlib import Path

def bge_m3_path(git_dir: os.PathLike):
    model_dir = clone(git_dir, repo_id="BAAI/bge-m3",allow_patterns=["onnx/*"])
    onnx_path = Path(model_dir) / "onnx" / "model.onnx"
    assert onnx_path.is_file() and onnx_path.exists()
    return onnx_path



from __future__ import annotations

import os
from pathlib import Path
from typing import Iterable, TypedDict, Callable, Any, Optional
from urllib.parse import urlparse

import requests

from davidkhala.ai.agent.dify.base import API


class DatasetDict(TypedDict):
    id: str
    name: str
    description: str
    provider: str
    permission: str
    data_source_type: str
    indexing_technique: str
    doc_form: str
    runtime_mode: str
    is_published: bool
    enable_api: bool
    # stats
    app_count: int
    document_count: int
    word_count: int
    total_documents: int
    total_available_documents: int
    # embedding
    embedding_available: bool
    embedding_model: str
    embedding_model_provider: str
    retrieval_model_dict: dict
    external_retrieval_model: dict
    external_knowledge_info: dict


class Iterator(Iterable):
    def __iter__(self):
        return self

    def __init__(self, get_fn: Callable[[int, int], Any], r: Optional[dict]):
        self.response = r
        self.fn = get_fn

    def __next__(self):
        if self.response and not self.response['has_more']:
            raise StopIteration
        page = 1 if not self.response else self.response['page'] + 1
        limit = None if not self.response else self.response['limit']
        self.response = self.fn(page, limit)
        return self.response['data']


class DocumentDict(TypedDict):
    id: str
    position: int
    data_source_type: str
    data_source_info: dict[str, str]
    data_source_detail_dict: dict[str, dict]
    dataset_process_rule_id: str
    name: str
    created_from: str
    created_by: str
    created_at: int
    tokens: int
    indexing_status: str
    error: str
    enabled: bool
    archived: bool
    display_status: str
    word_count: int
    hit_count: int
    doc_form: str
    doc_metadata: dict
    disabled_at: int
    disabled_by: str


class Dataset(API):
    def __init__(self, api_key: str, base_url="https://api.dify.ai/v1"):
        super().__init__(api_key, f"{base_url}/datasets")

    def paginate_datasets(self, page=1, size=20):
        r = self.request(self.base_url, "GET", params={
            'page': page,
            'limit': size,
        })
        return r

    def list_datasets(self) -> Iterable[list[DatasetDict]]:
        return Iterator(self.paginate_datasets, None)

    @property
    def ids(self):
        for sub_list in self.list_datasets():
            for dataset in sub_list:
                yield dataset['id']

    class Instance(API):
        def __init__(self, d: Dataset, dataset_id: str):
            super().__init__(d.api_key, f"{d.base_url}/{dataset_id}")

        def get(self):
            return self.request(self.base_url, "GET")

        def upload(self, filename, *, path=None, url=None, document_id=None):
            """
            don't work for html
            work for markdown
            TODO how to simulate console
            """
            files = {}
            if path:
                with open(path, 'rb') as f:
                    content = f.read()
                if not filename:
                    filename = os.path.basename(path)
            elif url:
                r = requests.get(url)
                r.raise_for_status()
                if not filename:
                    parsed_url = urlparse(url)
                    filename = Path(parsed_url.path).name
                content = r.content
            files['file'] = (filename, content)
            if document_id:
                # don't work for html
                r = requests.post(f"{self.base_url}/documents/{document_id}/update-by-file", files=files,
                                  **self.options)
            else:
                r = requests.post(f"{self.base_url}/document/create-by-file", files=files, **self.options)
            r = self.on_response(r)
            return r['document']

        def paginate_documents(self, page=1, size=20):
            return self.request(f"{self.base_url}/documents", "GET", params={
                'page': page,
                'limit': size
            })

        def list_documents(self) -> Iterable[DocumentDict]:
            for document_batch in Iterator(self.paginate_documents, None):
                for document in document_batch:
                    yield document

        def has_document(self, name) -> bool:
            return any(name == item['name'] for row in self.list_documents() for item in row)


class ChunkDict(TypedDict):
    id: str
    position: int
    document_id: str
    content: str
    sign_content: str  # trimmed version of content
    answer: Optional[str]  # only used in QA chunk
    word_count: int
    tokens: int
    keywords: Optional[list[str]]
    index_node_id: str  # chunk 在向量索引中的节点 ID
    index_node_hash: str  # hash of sign_content
    hit_count: int
    enabled: bool
    status: str  # 'completed'
    created_at: int  # timestamp
    updated_at: int  # timestamp
    completed_at: int  # timestamp
    created_by: str  # user id
    child_chunks: list
    error: Optional
    stopped_at: Optional[int]  # timestamp
    disabled_at: Optional[int]  # timestamp


class Document(API):
    def __init__(self, d: Dataset.Instance, document_id: str):
        super().__init__(d.api_key, f"{d.base_url}/documents/{document_id}")
        try:
            self.get()
            self.exist = True
        except requests.exceptions.HTTPError as e:
            if e.response.status_code == 404:
                self.exist = False
            else:
                raise e

    def get(self):
        return self.request(self.base_url, "GET")

    def paginate_chunks(self, page=1, size=20):
        return self.request(f"{self.base_url}/segments", "GET", params={
            'page': page,
            'limit': size
        })

    def list_chunks(self) -> Iterable[ChunkDict]:
        for chunk_batch in Iterator(self.paginate_chunks, None):
            for chunk in chunk_batch:
                yield chunk



import datetime
from abc import abstractmethod

from davidkhala.utils.http_request import Request

from davidkhala.ai.model import AbstractClient


class API(AbstractClient, Request):
    def __init__(self, api_key: str, base_url: str):
        super().__init__({
            "bearer": api_key
        })
        self.base_url = base_url + '/v1'

    @property
    @abstractmethod
    def free_models(self) -> list[str]:
        ...

    def chat(self, *user_prompt: str, **kwargs):
        messages = [
            *self.messages,
            *[{
                "role": "user",
                "content": _
            } for _ in user_prompt],
        ]

        json = {
            "messages": messages,
            **kwargs,
        }

        response = self.request(f"{self.base_url}/chat/completions", "POST", json=json)

        return {
            "data": list(map(lambda x: x['message']['content'], response['choices'])),
            "meta": {
                "usage": response['usage'],
                "created": datetime.datetime.fromtimestamp(response['created'])
            },
            'model': response['model'],
        }

    def list_models(self):
        response = self.request(f"{self.base_url}/models", "GET")
        return response['data']



import os
from typing import Optional

from huggingface_hub import snapshot_download


def clone(git_dir: os.PathLike,
          *,
          owner: Optional[str] = None,
          repository: Optional[str] = None,
          repo_id: Optional[str] = None,
          **kwargs
          ) -> str:
    if not repo_id:
        repo_id = f"{owner}/{repository}"
    return snapshot_download(
        repo_id=repo_id,
        local_dir=git_dir,
        local_dir_use_symlinks=False,
        **kwargs
    )



import runpy
from typing import Union, Literal

from openai import OpenAI, AsyncOpenAI

from davidkhala.ai.model import AbstractClient


class Client(AbstractClient):
    client: OpenAI
    encoding_format: Literal["float", "base64"] = "float"
    n = 1

    def connect(self):
        self.client.models.list()

    def encode(self, *_input: str) -> list[list[float]]:
        response = self.client.embeddings.create(
            model=self.model,
            input=list(_input),
            encoding_format=self.encoding_format
        )
        return [item.embedding for item in response.data]

    def chat(self, *user_prompt, **kwargs):

        messages = [
            *self.messages,
        ]
        for prompt in user_prompt:
            message = {
                "role": "user"
            }
            if type(prompt) == str:
                message['content'] = prompt
            elif type(prompt) == dict:
                message['content'] = [
                    {"type": "text", "text": prompt['text']},
                    {
                        "type": "image_url",
                        "image_url": {
                            "url": prompt['image_url'],
                        }
                    },
                ]
            messages.append(message)
        response = self.client.chat.completions.create(
            model=self.model,
            messages=messages,
            n=self.n,
            **kwargs
        )
        contents = [choice.message.content for choice in response.choices]
        assert len(contents) == self.n
        return contents

    def disconnect(self):
        self.client.close()


def with_opik(instance: Union[OpenAI, AsyncOpenAI]):
    from opik.integrations.openai import track_openai
    runpy.run_path('../opik.py')
    return track_openai(instance)
